@@ -14,9 +14,10 @@ def err(msg, die=None):
     if die:
         sys.exit((die if type(die) is int else 1))
 
+
 try:
     import pelican
-except:
+except ImportError:
     err('Cannot import pelican.\nYou must '
         'install Pelican in order to run this script.',
         -1)
@@ -80,9 +81,9 @@ def main():
     to_sym = args.to_symlink or args.clean
 
     if args.action:
-        if args.action is 'list':
+        if args.action == 'list':
             list_themes(args.verbose)
-        elif args.action is 'path':
+        elif args.action == 'path':
             print(_THEMES_PATH)
     elif to_install or args.to_remove or to_sym:
         if args.to_remove:
